@@ -586,6 +586,7 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 	ORBit_OAObject          adaptor_obj;
 	GIOPRecvBuffer         *recv_buffer = NULL;
 	CORBA_Object            xt_proxy = CORBA_OBJECT_NIL;
+	ORBitPolicy            *invoke_policy = CORBA_OBJECT_NIL;
 
 	if (!obj) {
 		dprintf (MESSAGES, "Cannot invoke method on null object\n");
@@ -594,6 +595,9 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 		goto clean_out;
 	}
 
+	if ((invoke_policy = ORBit_object_get_policy (obj)))
+		ORBit_policy_push (invoke_policy);
+
 	adaptor_obj = obj->adaptor_obj;
 
 	if (adaptor_obj) {
@@ -673,6 +677,10 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 	tprintf_end_method ();
 	if (cnx)
 		giop_connection_unref (cnx);
+	if (invoke_policy) {
+		ORBit_policy_pop ();
+		ORBit_policy_unref (invoke_policy);
+	}
 	return;
 
  system_exception:
@@ -933,18 +941,19 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 	giop_send_buffer_write (send_buffer, recv_buffer->connection, FALSE);
 	giop_send_buffer_unuse (send_buffer);
 
-	if (m_data->ret && tc->kind != CORBA_tk_void &&
-	    ev->_major == CORBA_NO_EXCEPTION) {
+	if (m_data->ret && tc->kind != CORBA_tk_void) {
 		switch (m_data->ret->kind) {
 		case BASE_TYPES:
 			break;
 		case CORBA_tk_objref:
 		case CORBA_tk_TypeCode:
-			CORBA_Object_release (*(CORBA_Object *) retval, ev);
+			if (ev->_major == CORBA_NO_EXCEPTION)
+				CORBA_Object_release (*(CORBA_Object *) retval, ev);
 			break;
 		case CORBA_tk_string:
 		case CORBA_tk_wstring:
-			ORBit_free (*(char **) retval);
+			if (ev->_major == CORBA_NO_EXCEPTION)
+				ORBit_free (*(char **) retval);
 			break;
 		case STRUCT_UNION_TYPES:
 			if (m_data->flags & ORBit_I_COMMON_FIXED_SIZE) {
@@ -952,7 +961,8 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 				break;
 			} /* drop through */
 		default:
-			ORBit_free (pretval);
+			if (ev->_major == CORBA_NO_EXCEPTION)
+				ORBit_free (pretval);
 			break;
 		}
 	}
@@ -993,20 +1003,30 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 				ORBit_free (args [i]);
 				break;
 			}
-		} else if (ev->_major == CORBA_NO_EXCEPTION) { /* Out */
+		} else { /* Out */
 			switch (tc->kind) {
 			case BASE_TYPES:
 				break;
 			case CORBA_tk_objref:
 			case CORBA_tk_TypeCode:
-				CORBA_Object_release (*(CORBA_Object *) scratch [i], ev);
+				if (ev->_major == CORBA_NO_EXCEPTION)
+					CORBA_Object_release (*(CORBA_Object *) scratch [i], ev);
 				break;
 			case CORBA_tk_string:
 			case CORBA_tk_wstring:
-				ORBit_free (*(char **) scratch [i]);
+				if (ev->_major == CORBA_NO_EXCEPTION)
+					ORBit_free (*(char **) scratch [i]);
 				break;
+			case STRUCT_UNION_TYPES:
+			case CORBA_tk_array:
+				if (a->flags & ORBit_I_COMMON_FIXED_SIZE) {
+					ORBit_free (scratch [i]);
+					break;
+				}
+				/* drop through */
 			default:
-				ORBit_free (scratch [i]);
+				if (ev->_major == CORBA_NO_EXCEPTION)
+					ORBit_free (scratch [i]);
 				break;
 			}
 		}
@@ -1114,6 +1134,7 @@ async_recv_cb (ORBitAsyncQueueEntry *aqe)
 
 	ORBit_RootObject_release (aqe->obj);
 /*	ORBit_RootObject_release (aqe->m_data); */
+	giop_recv_list_destroy_queue_entry (&aqe->mqe);
 	g_free (aqe);
 	CORBA_exception_free (ev);
 }
@@ -1280,7 +1301,9 @@ ORBit_small_listen_for_broken (CORBA_Object obj,
 
 		if (cnx) {
 			ret = get_status (cnx);
-			g_signal_connect (cnx, "broken", fn, user_data);
+			link_connection_add_broken_cb
+				(LINK_CONNECTION (cnx),
+				 (LinkBrokenCallback)fn, user_data);
 			giop_connection_unref (cnx);
 		} else
 			ret = ORBIT_CONNECTION_DISCONNECTED;
@@ -1308,17 +1331,10 @@ ORBit_small_unlisten_for_broken_full (CORBA_Object obj,
 		cnx = ORBit_object_get_connection (obj);
 
 		if (cnx) {
-			GSignalMatchType t = 0;
-
 			ret = get_status (cnx);
-			if (fn)
-				t |= G_SIGNAL_MATCH_FUNC;
-			if (user_data)
-				t |= G_SIGNAL_MATCH_DATA;
-
-			g_signal_handlers_disconnect_matched
-				(cnx, t, 0, 0, NULL,
-				 G_CALLBACK (fn), user_data);
+			link_connection_remove_broken_cb
+				(LINK_CONNECTION (cnx),
+				 (LinkBrokenCallback)fn, user_data);
 			giop_connection_unref (cnx);
 		} else
 			ret = ORBIT_CONNECTION_DISCONNECTED;
